@@ -23,12 +23,15 @@ __copyright__ = '(C) 2015, Luigi Pirelli'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '93e761dd2396af21db9bb93e26d7cba680c906ca'
+__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
 
 import codecs
 import xml.sax.saxutils
 
-from osgeo import ogr
+import warnings
+with warnings.catch_warnings():
+    warnings.filterwarnings("ignore", category=DeprecationWarning)
+    from osgeo import ogr
 from qgis.core import (QgsProcessingFeedback,
                        QgsProcessingParameterMultipleLayers,
                        QgsProcessingParameterBoolean,
