@@ -23,7 +23,7 @@ __copyright__ = '(C) 2012, Martin Dobias'
 
 # This will get replaced with a git SHA1 when you do a git archive
 
-__revision__ = '4a4b62ed19d2333b1f3a9e72bf77119048e3c9c0'
+__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
 
 import psycopg2
 import psycopg2.extensions  # For isolation levels
@@ -112,7 +112,7 @@ class TableConstraint(object):
     match_types = {'u': 'UNSPECIFIED', 'f': 'FULL', 'p': 'PARTIAL'}
 
     def __init__(self, row):
-        (self.name, con_type, self.is_defferable, self.is_deffered, keys) = row[:5]
+        (self.name, con_type, self.is_deferable, self.is_deferred, keys) = row[:5]
         self.keys = list(map(int, keys.split(' ')))
         self.con_type = TableConstraint.types[con_type]  # Convert to enum
         if self.con_type == TableConstraint.TypeCheck:
@@ -222,17 +222,26 @@ class GeoDB(object):
                 sslCertFile = expandedUri.param("sslcert")
                 if sslCertFile:
                     sslCertFile = sslCertFile.replace("'", "")
-                    os.remove(sslCertFile)
+                    try:
+                        os.remove(sslCertFile)
+                    except OSError:
+                        pass
 
                 sslKeyFile = expandedUri.param("sslkey")
                 if sslKeyFile:
                     sslKeyFile = sslKeyFile.replace("'", "")
-                    os.remove(sslKeyFile)
+                    try:
+                        os.remove(sslKeyFile)
+                    except OSError:
+                        pass
 
                 sslCAFile = expandedUri.param("sslrootcert")
                 if sslCAFile:
                     sslCAFile = sslCAFile.replace("'", "")
-                    os.remove(sslCAFile)
+                    try:
+                        os.remove(sslCAFile)
+                    except OSError:
+                        pass
 
         self.has_postgis = self.check_postgis()
 
